/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __CONF_HEADER_INCLUDED__
#define __CONF_HEADER_INCLUDED__

#include <g_globals.h>
#include <s_globals.h>
#include <lists.h>

typedef enum {
	KW_ERROR=0,	/*Broken args*/
	KW_OK,		/*Fine*/
	KW_MULTIPLE	/*This is a one off setting and i've seen it already*/
}kw_ret;

typedef kw_ret (*proc_keyword_handler)(int, char**);
typedef struct conf_kw {
	const char		*keyword;
	proc_keyword_handler	proc;
}conf_kw_t, *p_conf_kw;

void conf_cleanup_temp(void);
int conf_load_config(char *fn);
int conf_go(void);

int conf_parse_line(char *line); /* -1 on error, kw_ret on OK  */
void conf_cleanup_args(p_config_arg p);
proc_keyword_handler conf_get_handler(const char *kw);

extern conf_kw_t keywords[];

/* HANDLERS */
kw_ret confh_load_plugins(int, char**);
kw_ret confh_capdev_add(int, char**);
kw_ret confh_load_ruleset(int, char**);
kw_ret confh_firestorm_root(int, char **);
kw_ret confh_chroot(int, char **);
kw_ret confh_euid(int, char **);
kw_ret confh_egid(int, char **);
kw_ret confh_require(int, char **);
kw_ret confh_global(int, char **);

#define IS_CONFIG(x) ( \
	x[0] != '\0' && \
	x[0] != '#'  && \
	x[0] != '\n' && \
	x[0] != '\r' )

#endif //__CONF_HEADER_INCLUDED__
