/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __PACKET_HEADER_INCLUDED__
#define __PACKET_HEADER_INCLUDED__

#include <g_globals.h>
#include <pkt/ip.h>
#include <pkt/icmp.h>
#include <pkt/igmp.h>
#include <pkt/tcp.h>
#include <pkt/udp.h>
#include <pkt/arp.h>
#include <pkt/gre.h>
#include <capdev.h>

#define PORT ushort
#define CAP_SNAPLEN 4096

/* Max physically possible is 255 */
#define MAX_DEMUX_LAYERS	16

/* From the linux kernel (ish) */
struct ethhdr{
	unsigned char  dst[6];	/* destination eth addr */
	unsigned char  src[6];	/* source ether addr    */
	unsigned short proto;	/* packet type ID field */
};

typedef enum layer_type {
	LTYPE_LINK,
	LTYPE_NET,
	LTYPE_TRANS,
	LTYPE_APP,
	LTYPE_ENCAP,
	LTYPE_MAX
}layer_type_t;

typedef struct layer {
	layer_type_t 	type;
	u_int32_t 	id;
	char 		*name;
	union {
		void	*raw;
		p_ip	ip;
		p_icmp	icmp;
		p_igmp	igmp;
		p_tcp	tcp;
		p_udp	udp;
		p_gre	gre;
		struct ethhdr *mac;
	}h;
}layer_t, *p_layer;

typedef struct packet {
	time_t		time;	/* Timestamp */
	time_t		nano_time; /* Extra space for more accurate time */
	struct tm	ltime;	/* Localised timestamp */
	
	uint		caplen;	/* Length of present data */
	uint		len;	/* Real length of data on the wire */

	p_capdev	capdev; /* The capture device that cought it */
	
	/* This is as much voodoo magic as you will ever see
	* in the firestorm core, but ports are common cocepts
	* to quite a lot of protocols */
	PORT		sport, dport; /* Source and dest ports */

	/* Index in to layers array of certain types 
	* NOTE: this kinda sucks, as soon as we can
	* express rules in a nicer way, kill this */
	u_int8_t	top_layer;
	u_int8_t	top_link;
	u_int8_t	top_net;
	u_int8_t	top_trans;
	u_int8_t	top_app;
	u_int8_t	top_encap;
	
	u_int16_t	reserved; /* Padding, may be used in future */
	
	layer_t         layer[MAX_DEMUX_LAYERS]; /* Network layers */
	u_int8_t	data[CAP_SNAPLEN]; /* Full packet data */
}packet_t, *p_packet;

#endif //__PACKET_HEADER_INCLUDED__
